import cf
import tempfile
import os
import sys
import numpy

'''
Test the cf package:

TEST: Create a field in memory
TEST: Print a dump of the field
TEST: Write the field to disk
TEST: Read the field from disk, with data stored as netCDF4.Variable objects
TEST: Access the field's data as a numpy array
TEST: Access the field's coordinates' data as a numpy arrays
TEST: Slice the field
TEST: Check the equality function
TEST: Create list of fields
TEST: Write a list of fields to disk
TEST: Read a list of fields from disk
TEST: Print all last fields in the list
TEST: Print the last field in the list
TEST: Print the data of the last field in the list
TEST: Modify the last field in the list
'''

print '\nTEST: Create a field in memory:'

f = cf.Field()
f.space = cf.Space()
 
f.standard_name = 'test_field'
f.units         = 'm s-1'

f.space['dim0'] = cf.Coordinate()
f.space['dim0'].standard_name = 'latitude'
f.space['dim0'].units         = 'degrees_north'
f.space['dim0']._data = cf.Data(numpy.arange(10, dtype = float))
 
f.space['dim1'] = cf.Coordinate()
f.space['dim1'].standard_name = 'longitude'
f.space['dim1'].units         = 'degrees_east'
f.space['dim1']._data = cf.Data(numpy.arange(9, dtype = float) + 20)

f.space.dimensions = {'data': ['dim0', 'dim1'],
                      'dim0': ['dim0'],
                      'dim1': ['dim1'],
                      }

f.space.dimension_sizes = {'dim0': 10,
                           'dim1': 9,
                          }

f._data = cf.Data(numpy.arange(90, dtype = float).reshape(10,9))

print '\nTEST: Print a dump of the field:'
print repr(f)

cf.dump(f)

print '\nTEST: Write the field to disk:'
file = tempfile.mktemp('.nc')

cf.write(f, file, verbose=1)

print '\nTEST: Read the field from disk, with data stored as netCDF4.Variable objects: '
g = cf.read(file, verbose=1)[0]
g.delpub('history')

cf.dump(g)

print '\nTEST: Access the field\'s data as a numpy array:'
print g.varray

print '\nTEST: Access the field\'s coordinates\' data as a numpy arrays:'

print g.coord('lat').varray
print g.coord('lon').varray

print '\nTEST: Slice the field:'
print g.slice(lat = cf.lt(5), lon = 27)

print '\nTEST: Slice the field:'
print g.slice[...,2:5]

print '\nTEST: Check the equality function:'
result  = cf.equals(g, g)

if not result:
    raise RuntimeError('Field is not equal to itself')

print '\nTEST: Create list of fields:'
fl = cf.FieldList([g, g, g, g])

print '\nTEST: Write a list of fields to disk:'
cf.write(fl, file, verbose=1)

print '\nTEST: Read a list of fields from disk:'
fl = cf.read(file, verbose=1)
fl.delpub('history')
print repr(fl)

print '\nTEST: Print all last fields in the list:'
print fl

print '\nTEST: Print the last field in the list:'
print fl[-1]

print '\nTEST: Print the data of the last field in the list:'
print fl[-1].varray

print '\nTEST: Modify the last field in the list:'
fl[-1] *= -1
print fl[-1].varray

print '\n---------------------------------------------'
print   'TEST: All tests passed for cf version',cf.__version__
print   '--------------------------------------------- '

# Tidy up
os.remove(file)

